# external dependencies
require 'nokogiri'
require 'pdfkit'
require 'baby_erubis'
require 'active_support'
require 'active_support/core_ext'

# stdlib dependencies
require 'date'
require 'nkf'
require 'uri'
require 'open-uri'
require 'net/http'

module QiitaPicks
  class Picker
    ERB_TEMPLATE_ARTICLE = File.expand_path '../templates/article.erb', __FILE__
    ERB_TEMPLATE_LAYOUT  = File.expand_path '../templates/layout.erb', __FILE__
    PDFKIT_OPTIONS = {
      encoding:      'UTF-8',
      page_size:     'A4',
      quiet:         false,
      no_background: false
    }.freeze

    attr_reader :path_to_pdf, :path_to_html

    def self.hatebu_bookmark_url(tag=nil, sort=:popular)
      uri = URI('http://b.hatena.ne.jp/search/text')
      uri.query = {
        q: ['http://qiita.com/', tag].compact.join(' '),
        date_begin: (Time.now - 60*60*24*30).strftime('%Y-%m-%d'),
        date_end: Time.now.strftime('%Y-%m-%d'),
        users: 3,
        sort: sort,
        safe: 'off',
        mode: 'rss'
      }.to_param
      doc = Nokogiri::XML open(uri.to_s)
      doc.xpath("//rdf:Seq/rdf:li").map{|x| x.attr("rdf:resource")}
    end

    def build_html_from_url(url=[], title=nil)
      return if url.empty?
      @title = title || Time.now.strftime(QiitaPicks::EMAIL_SUBJECT_DEFAULT)
      @contents ||= build_articles_from_url url
      @html     ||= html
    end

    def build_html_from_items(items=[], title=nil)
      return if items.empty?
      @title = title || Time.now.strftime(QiitaPicks::EMAIL_SUBJECT_DEFAULT)
      @contents ||= build_articles_from_items items
      @html     ||= html
    end

    def save_as_pdf(opts={})
      return if @html.nil?
      opts[:dest]        ||= '.'
      opts[:basename]    ||= Time.now.strftime QiitaPicks::FILE_BASENAME_DEFAULT
      opts[:pdf_options] ||= {}
      @path_to_pdf = File.join(opts[:dest], "#{opts[:basename]}.pdf")
      PDFKit.new(@html, PDFKIT_OPTIONS.merge(opts[:pdf_options])).to_file @path_to_pdf
      puts "[GENERATE] #{@path_to_pdf}"
    end

    def save_as_html(opts={})
      return if @html.nil?
      opts[:dest]     ||= '.'
      opts[:basename] ||= Time.now.strftime QiitaPicks::FILE_BASENAME_DEFAULT
      @path_to_html = File.join(opts[:dest], "#{opts[:basename]}.html")
      open(@path_to_html, "w+") {|f| f.write @html}
      puts "[GENERATE] #{@path_to_html}"
    end

    private

    def html
      return if @contents.empty?
      template = BabyErubis::Html.new.from_file(ERB_TEMPLATE_LAYOUT, 'utf-8')
      template.render(title: @title, contents: @contents, credit: HTML_FOOTER_CREDIT)
    end

    def build_articles_from_url(urls=[])
      urls.select {|url| /qiita/ =~ URI.parse(url).host}.uniq.inject('') {|articles, url|
        if res = fetch(url)
          uri = res.uri
          doc = Nokogiri::HTML NKF.nkf('-w', res.body)
          template = BabyErubis::Html.new.from_file(ERB_TEMPLATE_ARTICLE, 'utf-8')
          context  = {
            url:        "#{uri.scheme}://#{uri.host}#{uri.path}",
            title:      doc.css('title').text,
            content:    doc.css('.markdownContent').inner_html,
            user_name:  doc.css('.itemsShowAuthorInfo_userName').text,
            created_at: doc.css('ul.itemsShowHeaderTitle_status time').first.text
          }
          articles << template.render(context)
        end
        articles
      }
    end

    def build_articles_from_items(items=[])
      items.inject('') {|articles, item|
        template = BabyErubis::Html.new.from_file(ERB_TEMPLATE_ARTICLE, 'utf-8')
        context  = {
          url:        item['url'],
          title:      item['title'],
          content:    item['rendered_body'],
          user_name:  item['user']['id'],
          created_at: DateTime.parse(item['created_at']).strftime('%Y/%m/%d')
        }
        articles << template.render(context)
      }
    end

    def fetch(url, limit=10)
      res = Net::HTTP.get_response URI.parse(url)
      puts "[#{res.code} #{res.message}] #{url}"
      case res
      when Net::HTTPSuccess
        return res
      when Net::HTTPRedirection
        return fetch(res['location'], limit - 1)
      end
    end
  end
end
